use std::path::PathBuf;

use libc::gethostname;
use users::{get_current_gid, get_current_username, Group};
use which::which;

// use crate::error::RudoError;

pub fn get_command_path(command: &str) -> Result<PathBuf, String> {
    if let Ok(path) = which(command) {
        Ok(path)
    } else {
        let mut path = PathBuf::new();
        path.push(command.to_owned());
        Ok(path)
    }
}

pub fn get_username() -> Result<String, String> {
    let bad_username = String::from("Error: only UTF-8 usernames are supported.");
    if let Some(username) = get_current_username() {
        if let Ok(username) = username.into_string() {
            Ok(username)
        } else {
            Err(bad_username)
        }
    } else {
        Err(bad_username)
    }
}

pub fn get_groups() -> Vec<Group> {
    if let Some(groups) =
        users::get_user_groups(&get_username().unwrap_or_default(), get_current_gid())
    {
        groups
    } else {
        vec![]
    }
}

pub fn get_hostname() -> Result<String, String> {
    let bad_hostname = String::from("Error: only UTF-8 hostnames are supported.");
    let mut hostname = [0u8; 255];
    unsafe {
        if gethostname(hostname.as_mut_ptr() as *mut i8, hostname.len()) != 0 {
            return Err(bad_hostname);
        }
    };
    if let Ok(s) = String::from_utf8(hostname.to_vec()) {
        let s = s.trim_matches(char::from_u32(0).unwrap()).to_owned();
        Ok(s)
    } else {
        Err(bad_hostname)
    }
}
